
#ifndef AKONADIAGENTBASE_EXPORT_H
#define AKONADIAGENTBASE_EXPORT_H

#ifdef AKONADIAGENTBASE_STATIC_DEFINE
#  define AKONADIAGENTBASE_EXPORT
#  define AKONADIAGENTBASE_NO_EXPORT
#else
#  ifndef AKONADIAGENTBASE_EXPORT
#    ifdef KPim6AkonadiAgentBase_EXPORTS
        /* We are building this library */
#      define AKONADIAGENTBASE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define AKONADIAGENTBASE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef AKONADIAGENTBASE_NO_EXPORT
#    define AKONADIAGENTBASE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef AKONADIAGENTBASE_DEPRECATED
#  define AKONADIAGENTBASE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef AKONADIAGENTBASE_DEPRECATED_EXPORT
#  define AKONADIAGENTBASE_DEPRECATED_EXPORT AKONADIAGENTBASE_EXPORT AKONADIAGENTBASE_DEPRECATED
#endif

#ifndef AKONADIAGENTBASE_DEPRECATED_NO_EXPORT
#  define AKONADIAGENTBASE_DEPRECATED_NO_EXPORT AKONADIAGENTBASE_NO_EXPORT AKONADIAGENTBASE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef AKONADIAGENTBASE_NO_DEPRECATED
#    define AKONADIAGENTBASE_NO_DEPRECATED
#  endif
#endif

#endif /* AKONADIAGENTBASE_EXPORT_H */
