/*
    SPDX-FileCopyrightText: 2009 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once
#include "akonadiwidgets_export.h"
class QSqlDatabase;

namespace DbAccess
{
AKONADIWIDGETS_EXPORT QSqlDatabase database();
}
